@@ -1,27 +1,13 @@
-/*=============================================================================
-
-    This file is part of ARB.
-
-    ARB is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    ARB is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with ARB; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
-
-=============================================================================*/
-/******************************************************************************
-
+/*
     Copyright (C) 2012 Fredrik Johansson
 
-******************************************************************************/
+    This file is part of Arb.
+
+    Arb is free software: you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 2.1 of the License, or
+    (at your option) any later version.  See <http://www.gnu.org/licenses/>.
+*/
 
 #ifndef ARB_POLY_H
 #define ARB_POLY_H
@@ -32,10 +18,11 @@
 #define ARB_POLY_INLINE static __inline__
 #endif
 
-#include "arb.h"
-#include "acb.h"
+#include <stdio.h>
 #include "flint/fmpz_poly.h"
 #include "flint/fmpq_poly.h"
+#include "arb.h"
+#include "acb.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -78,6 +65,10 @@ void arb_poly_set(arb_poly_t poly, const arb_poly_t src);
 
 void arb_poly_set_round(arb_poly_t poly, const arb_poly_t src, slong prec);
 
+void arb_poly_set_trunc(arb_poly_t res, const arb_poly_t poly, slong n);
+
+void arb_poly_set_trunc_round(arb_poly_t res, const arb_poly_t poly, slong n, slong prec);
+
 /* Basic manipulation */
 
 ARB_POLY_INLINE slong arb_poly_length(const arb_poly_t poly)
@@ -90,6 +81,27 @@ ARB_POLY_INLINE slong arb_poly_degree(const arb_poly_t poly)
     return poly->length - 1;
 }
 
+slong arb_poly_valuation(const arb_poly_t poly);
+
+ARB_POLY_INLINE int
+arb_poly_is_zero(const arb_poly_t z)
+{
+    return arb_poly_length(z) == 0;
+}
+
+ARB_POLY_INLINE int
+arb_poly_is_one(const arb_poly_t z)
+{
+    return (arb_poly_length(z) == 1) && arb_is_one(z->coeffs);
+}
+
+ARB_POLY_INLINE int
+arb_poly_is_x(const arb_poly_t z)
+{
+    return (arb_poly_length(z) == 2) && arb_is_zero(z->coeffs)
+        && arb_is_one(z->coeffs + 1);
+}
+
 ARB_POLY_INLINE void arb_poly_zero(arb_poly_t poly)
 {
     poly->length = 0;
@@ -175,7 +187,13 @@ void arb_poly_majorant(arb_poly_t res, const arb_poly_t poly, slong prec);
 
 /* IO */
 
-void arb_poly_printd(const arb_poly_t poly, slong digits);
+void arb_poly_fprintd(FILE * file, const arb_poly_t poly, slong digits);
+
+ARB_POLY_INLINE void
+arb_poly_printd(const arb_poly_t poly, slong digits)
+{
+    arb_poly_fprintd(stdout, poly, digits);
+}
 
 /* Random generation */
 
@@ -198,6 +216,12 @@ void _arb_poly_sub(arb_ptr res, arb_srcptr poly1, slong len1,
 void arb_poly_sub(arb_poly_t res, const arb_poly_t poly1,
               const arb_poly_t poly2, slong prec);
 
+void arb_poly_add_series(arb_poly_t res, const arb_poly_t poly1,
+              const arb_poly_t poly2, slong len, slong prec);
+
+void arb_poly_sub_series(arb_poly_t res, const arb_poly_t poly1,
+              const arb_poly_t poly2, slong len, slong prec);
+
 ARB_POLY_INLINE void
 arb_poly_neg(arb_poly_t res, const arb_poly_t poly)
 {
@@ -214,6 +238,24 @@ arb_poly_scalar_mul_2exp_si(arb_poly_t res, const arb_poly_t poly, slong c)
     _arb_poly_set_length(res, poly->length);
 }
 
+ARB_POLY_INLINE void
+arb_poly_scalar_mul(arb_poly_t res, const arb_poly_t poly, const arb_t c, slong prec)
+{
+    arb_poly_fit_length(res, poly->length);
+    _arb_vec_scalar_mul(res->coeffs, poly->coeffs, poly->length, c, prec);
+    _arb_poly_set_length(res, poly->length);
+    _arb_poly_normalise(res);
+}
+
+ARB_POLY_INLINE void
+arb_poly_scalar_div(arb_poly_t res, const arb_poly_t poly, const arb_t c, slong prec)
+{
+    arb_poly_fit_length(res, poly->length);
+    _arb_vec_scalar_div(res->coeffs, poly->coeffs, poly->length, c, prec);
+    _arb_poly_set_length(res, poly->length);
+    _arb_poly_normalise(res);
+}
+
 void _arb_poly_mullow_ztrunc(arb_ptr res,
     arb_srcptr poly1, slong len1,
     arb_srcptr poly2, slong len2, slong n, slong prec);
@@ -303,6 +345,22 @@ void _arb_poly_tree_build(arb_ptr * tree, arb_srcptr roots, slong len, slong pre
 
 /* Composition */
 
+void _arb_poly_taylor_shift_horner(arb_ptr poly, const arb_t c, slong n, slong prec);
+
+void arb_poly_taylor_shift_horner(arb_poly_t g, const arb_poly_t f, const arb_t c, slong prec);
+
+void _arb_poly_taylor_shift_divconquer(arb_ptr poly, const arb_t c, slong n, slong prec);
+
+void arb_poly_taylor_shift_divconquer(arb_poly_t g, const arb_poly_t f, const arb_t c, slong prec);
+
+void _arb_poly_taylor_shift_convolution(arb_ptr poly, const arb_t c, slong n, slong prec);
+
+void arb_poly_taylor_shift_convolution(arb_poly_t g, const arb_poly_t f, const arb_t c, slong prec);
+
+void _arb_poly_taylor_shift(arb_ptr poly, const arb_t c, slong n, slong prec);
+
+void arb_poly_taylor_shift(arb_poly_t g, const arb_poly_t f, const arb_t c, slong prec);
+
 void _arb_poly_compose(arb_ptr res,
     arb_srcptr poly1, slong len1,
     arb_srcptr poly2, slong len2, slong prec);
@@ -486,9 +544,11 @@ void _arb_poly_sqrt_series(arb_ptr g,
 void arb_poly_sqrt_series(arb_poly_t g, const arb_poly_t h, slong n, slong prec);
 
 void _arb_poly_log_series(arb_ptr res, arb_srcptr f, slong flen, slong n, slong prec);
-
 void arb_poly_log_series(arb_poly_t res, const arb_poly_t f, slong n, slong prec);
 
+void _arb_poly_log1p_series(arb_ptr res, arb_srcptr f, slong flen, slong n, slong prec);
+void arb_poly_log1p_series(arb_poly_t res, const arb_poly_t f, slong n, slong prec);
+
 void _arb_poly_atan_series(arb_ptr res, arb_srcptr f, slong flen, slong n, slong prec);
 
 void arb_poly_atan_series(arb_poly_t res, const arb_poly_t f, slong n, slong prec);
@@ -501,15 +561,24 @@ void _arb_poly_acos_series(arb_ptr res, arb_srcptr f, slong flen, slong n, slong
 
 void arb_poly_acos_series(arb_poly_t res, const arb_poly_t f, slong n, slong prec);
 
-void _arb_poly_exp_series_basecase(arb_ptr f,
-        arb_srcptr h, slong hlen, slong n, slong prec);
-
+void _arb_poly_exp_series_basecase(arb_ptr f, arb_srcptr h, slong hlen, slong n, slong prec);
 void arb_poly_exp_series_basecase(arb_poly_t f, const arb_poly_t h, slong n, slong prec);
-
 void _arb_poly_exp_series(arb_ptr f, arb_srcptr h, slong hlen, slong n, slong prec);
-
 void arb_poly_exp_series(arb_poly_t f, const arb_poly_t h, slong n, slong prec);
 
+void _arb_poly_sinh_cosh_series_basecase(arb_ptr s, arb_ptr c, arb_srcptr h, slong hlen, slong n, slong prec);
+void arb_poly_sinh_cosh_series_basecase(arb_poly_t s, arb_poly_t c, const arb_poly_t h, slong n, slong prec);
+void _arb_poly_sinh_cosh_series_exponential(arb_ptr s, arb_ptr c, arb_srcptr h, slong hlen, slong n, slong prec);
+void arb_poly_sinh_cosh_series_exponential(arb_poly_t s, arb_poly_t c, const arb_poly_t h, slong n, slong prec);
+void _arb_poly_sinh_cosh_series(arb_ptr s, arb_ptr c, arb_srcptr h, slong hlen, slong n, slong prec);
+void arb_poly_sinh_cosh_series(arb_poly_t s, arb_poly_t c, const arb_poly_t h, slong n, slong prec);
+
+void _arb_poly_sinh_series(arb_ptr s, arb_srcptr h, slong hlen, slong n, slong prec);
+void arb_poly_sinh_series(arb_poly_t s, const arb_poly_t h, slong n, slong prec);
+
+void _arb_poly_cosh_series(arb_ptr c, arb_srcptr h, slong hlen, slong n, slong prec);
+void arb_poly_cosh_series(arb_poly_t c, const arb_poly_t h, slong n, slong prec);
+
 void _arb_poly_sin_cos_series_basecase(arb_ptr s,
                                     arb_ptr c, arb_srcptr h, slong hlen, slong n, slong prec, int times_pi);
 
@@ -517,19 +586,19 @@ void arb_poly_sin_cos_series_basecase(arb_poly_t s, arb_poly_t c,
         const arb_poly_t h, slong n, slong prec, int times_pi);
 
 void _arb_poly_sin_cos_series_tangent(arb_ptr s, arb_ptr c,
-                        const arb_srcptr h, slong hlen, slong len, slong prec, int times_pi);
+                        arb_srcptr h, slong hlen, slong len, slong prec, int times_pi);
 
 void arb_poly_sin_cos_series_tangent(arb_poly_t s, arb_poly_t c,
                                     const arb_poly_t h, slong n, slong prec, int times_pi);
 
 void _arb_poly_sin_cos_series(arb_ptr s, arb_ptr c,
-                        const arb_srcptr h, slong hlen, slong len, slong prec);
+                        arb_srcptr h, slong hlen, slong len, slong prec);
 
 void arb_poly_sin_cos_series(arb_poly_t s, arb_poly_t c,
                                     const arb_poly_t h, slong n, slong prec);
 
 void _arb_poly_sin_cos_pi_series(arb_ptr s, arb_ptr c,
-                        const arb_srcptr h, slong hlen, slong len, slong prec);
+                        arb_srcptr h, slong hlen, slong len, slong prec);
 
 void arb_poly_sin_cos_pi_series(arb_poly_t s, arb_poly_t c,
                                     const arb_poly_t h, slong n, slong prec);
@@ -558,6 +627,9 @@ void _arb_poly_tan_series(arb_ptr g, arb_srcptr h, slong hlen, slong len, slong
 
 void arb_poly_tan_series(arb_poly_t g, const arb_poly_t h, slong n, slong prec);
 
+void _arb_poly_sinc_series(arb_ptr g, arb_srcptr h, slong hlen, slong n, slong prec);
+void arb_poly_sinc_series(arb_poly_t g, const arb_poly_t h, slong n, slong prec);
+
 void _arb_poly_compose_series_brent_kung(arb_ptr res, arb_srcptr poly1, slong len1,
                             arb_srcptr poly2, slong len2, slong n, slong prec);
 
@@ -634,6 +706,12 @@ void _arb_poly_root_bound_fujiwara(mag_t bound, arb_srcptr poly, slong len);
 
 void arb_poly_root_bound_fujiwara(mag_t bound, arb_poly_t poly);
 
+ARB_POLY_INLINE slong
+arb_poly_allocated_bytes(const arb_poly_t x)
+{
+    return _arb_vec_allocated_bytes(x->coeffs, x->alloc);
+}
+
 /* Macros */
 
 
@@ -652,6 +730,9 @@ n_zerobits(mp_limb_t e)
     return zeros;
 }
 
+/* Computes the length of the result when raising a polynomial of
+   length *len* to the power *exp* and truncating to length *trunc*,
+   without overflow. Assumes poly_len >= 1. */
 ARB_POLY_INLINE slong
 poly_pow_length(slong poly_len, ulong exp, slong trunc)
 {
