@@ -1,32 +1,18 @@
-/*=============================================================================
-
-    This file is part of ARB.
-
-    ARB is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    ARB is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with ARB; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
-
-=============================================================================*/
-/******************************************************************************
-
+/*
     Copyright (C) 2012 Fredrik Johansson
 
-******************************************************************************/
+    This file is part of Arb.
+
+    Arb is free software: you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 2.1 of the License, or
+    (at your option) any later version.  See <http://www.gnu.org/licenses/>.
+*/
 
 #ifndef FMPR_H
 #define FMPR_H
 
-#if defined(__MINGW64__)
+#if defined(__MINGW64__) || defined(_MSC_VER)
 #include "stdint.h"
 #endif
 #include <stdio.h>
@@ -36,17 +22,15 @@
 #include "flint/flint.h"
 #include "flint/fmpz.h"
 #include "flint/fmpq.h"
+#include "flint/config.h"
 #include "fmpz_extras.h"
 
-#include "flint/config.h"
-#ifdef HAVE_TLS
-#if HAVE_TLS
-#define TLS_PREFIX __thread
+#define TLS_PREFIX FLINT_TLS_PREFIX
+
+#if defined(_MSC_VER) && defined(ARB_BUILD_DLL)
+#define ARB_DLL __declspec(dllexport)
 #else
-#define TLS_PREFIX
-#endif
-#else
-#define TLS_PREFIX
+#define ARB_DLL FLINT_DLL
 #endif
 
 #define fmpr_rnd_t int
@@ -60,6 +44,9 @@
 extern "C" {
 #endif
 
+/* currently defined in the arb module, but global to the library */
+double arb_test_multiplier(void);
+
 static __inline__ int
 rounds_up(fmpr_rnd_t rnd, int negative)
 {
